/* THIS SAMPLE CODE IS PROVIDED AS IS AND IS SUBJECT TO ALTERATIONS. FUJITSU */
/* MICROELECTRONICS ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR */
/* ELIGIBILITY FOR ANY PURPOSES                                              */
/*                 (C) Fujitsu Microelectronics Europe GmbH                  */
/* $Id: lowlevel.c,v 1.1 2003/02/03 16:00:25 dfisch Exp $ */

#include "MB90540.h"

/****************************** Prototyps ************************************/
int write(int , char *, unsigned int);
int read(int , char *, int);
int close(int);

/****************************** Procedures ***********************************/


/*****************************************************************************
 *  DESCRIPTION:    low level write function, transmit buffer content via
 *                  UART1, line feed (ASCII:10) is translated into line feed
 *                  and carrige returmn (ASCII: 10 and 13)
 *
 *  PARAMETERS:     file number, buffer pointer, number of characters to be 
 *                  written
 *
 *  RETURNS:        successfull: number of written characters
 *                  error: -1
 *****************************************************************************/
int write(int fileno, char *buf, unsigned int size)
{
	unsigned int cnt;
    unsigned char helpchar;
    	
	switch(fileno)
	{
		case 0 :	return(0);						/* stdin */
		case 1 :	for(cnt = 0;size > cnt;cnt++)	/* stdout */
					{
                      helpchar = *buf++;
                      if (helpchar == 10) {
	                      while (!SSR1_TDRE);
    	                  SODR1 = helpchar;
        	              helpchar = 13;
                        }
                      while (!SSR1_TDRE);
                      SODR1 = helpchar;
  					}
					return(size);					/* successful output */
		case 2 :	return(-1);						/* stderr */
		default:	return(-1);						/* should never happend */;
	}
}


/*****************************************************************************
 *  DESCRIPTION:    low level read function, read characters via UART1
 *                  carrige return (ASCII:13) is translated into line feed
 *                  and carrige returmn (ASCII: 10 and 13)
 *
 *  PARAMETERS:     file number, buffer pointer, maximal buffer size
 *
 *  RETURNS:        successfull: number of read characters
 *                  error: -1
 *****************************************************************************/
int read( int fileno, char *buf, int size) {

	unsigned int cnt;
    unsigned char helpchar;


	switch(fileno)
	{


		case 0 :	for(cnt = 0; cnt < size; cnt++)	/* stdin */
					{

                      while(!SSR1_RDRF && !SSR1_ORE); /* wait for data in buffer */
                      if (SSR1_ORE || SSR1_PE) {      /* overrun or parity error */
                        SCR1_REC = 0;                 /* reset error flags */
                        return (-1);
                      } else {
                        helpchar=SIDR1;

                        if (helpchar == 13) {
                          *buf=10;
		       		      return(cnt+1);			/* successful input */
                        }
                        *buf=helpchar;
                        buf++;
  					  }
  					}
					return(cnt);					/* successful input */
		case 1 :	return(0);						/* stdout */
		case 2 :	return(-1);						/* stderr */
		default:	return(-1);						/* should never happend */;
	}
}


/*****************************************************************************
 *  DESCRIPTION:    low level function
 *
 *  PARAMETERS:     file number
 *
 *  RETURNS:        successfull:0 , error: -1
 *****************************************************************************/
int close(int fileno)
{
	if((fileno >= 0) && (fileno <=2))
	{
		return(0);
	}
	else
		return(-1);
}

/*****************************************************************************/
/*EOF $Id: lowlevel.c,v 1.1 2003/02/03 16:00:25 dfisch Exp $ */
