/* THIS SAMPLE CODE IS PROVIDED AS IS AND IS SUBJECT TO ALTERATIONS. FUJITSU */
/* MICROELECTRONICS ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR */
/* ELIGIBILITY FOR ANY PURPOSES.                                             */
/*                 (C) Fujitsu Microelectronics Europe GmbH                  */
/* $Id: uart1.c,v 1.1 2003/02/03 16:00:25 dfisch Exp $ */

#include "mb90540.h"

/*****************************************************************************
 *  DESCRIPTION:    Initializes UART1 for asyncronous point-to-point 
 *                  communication mode, assumption: coreclk=16MHz
 *
 *  PARAMETERS:     baudr(1200..38400),databits (7,8),
 *                  parity (0=no, 1=odd, 2=even), stopbit(1,2)
 *
 *  RETURNS:        None
 *****************************************************************************/
void UART1init( long baudr, unsigned char datab,  unsigned char par,
                unsigned char stopb)
{
  /* Baudrate setting : */
  switch (baudr)
    {
    case 1200   : SMR1_CS=3; U1CDCR=8 ; break;	/* 1200 Baud */
    case 2400   : SMR1_CS=2; U1CDCR=8 ; break;	/* 2400 Baud */
    case 4800   : SMR1_CS=1; U1CDCR=8 ; break;	/* 4800 Baud */
    case 19200  : SMR1_CS=1; U1CDCR=14 ; break;	/* 19200 Baud */
    case 38400  : SMR1_CS=0; U1CDCR=14 ; break;	/* 38400 Baud */
    case 57600  :		/* 57600 Baud only with reload timer*/
    case 115200 :		/* 115200 Baud only with reload timer */
    case 9600   :
    default 	:SMR1_CS=0; U1CDCR=8 ; break;	/* 9600 Baud default*/
    }
  /* Number of Databits : */
  switch (datab)
  {
    case 7 :	SCR1_CL  = 0; break; /* 7 Databits */
    case 8 :	SCR1_CL  = 1; break; /* 8 Databits */
    default :	SCR1_CL  = 1; break; /* Default 8 Databits */
  }
  /* Number of Stopbits : */
  switch (stopb)
  {
    case 1 :	SCR1_SBL = 0; break; /* 1 Stopbit */
    case 2 :	SCR1_SBL = 1; break; /* 2 Stopbits */
    default :	SCR1_SBL = 0; break; /* Default 1 Stopbit */
  }
  /* Parity Select : */
  switch (par)
  {
    case 0 :	SCR1_PEN = 0; break; /* no parity */
    case 1 :	SCR1_PEN = 1; SCR1_P = 1; break; /* odd parity */
    case 2 :	SCR1_PEN = 1; SCR1_P = 0; break; /* even parity */
    default :	SCR1_PEN = 0; break; /* Default no parity */
  }

	DDR4_D45 = 1;       /* Make SOT1 working */
	SMR1_SOE = 1;       /* Make SOT1 working */
    SMR1_MD = 0;		/* async normal mode */
    SCR1_RXE = 1;		/* Receive enable */
    SCR1_TXE = 1;		/* Transmit enable */
    U1CDCR_MD = 1;		/* prescaler control register */
  
}


/*****************************************************************************
 *  DESCRIPTION:    sends a character via UART1
 *
 *  PARAMETERS:     character to be transmitted
 *
 *  RETURNS:        void
 *****************************************************************************/
void UART1putch (char ch)         /* sends a char */
{
	while (SSR1_TDRE == 0);	/* wait for transmit buffer empty 	*/
	 SODR1 = ch;		/* put ch into buffer			*/
	/*return ch;		 return transmitted character		*/
}

/*****************************************************************************
 *  DESCRIPTION:    waits for and returns incomming char
 *
 *  PARAMETERS:     void
 *
 *  RETURNS:        received character 
 *****************************************************************************/
char UART1getch(void)
{
  unsigned ch;
	while(SSR1_RDRF == 0);	/* wait for data received  	*/
	if (SSR1_ORE || SSR1_FRE)      	/* overrun error or frame error		*/
	{
		ch = SIDR1;	/* reset error flags 		*/
		return (-1);
	}
	else
		return (SIDR1);	/* return char 			*/
}

/*****************************************************************************
 *  DESCRIPTION:    Puts a String to UART
 *
 *  PARAMETERS:     string to be transmitted
 *
 *  RETURNS:        void
 *****************************************************************************/
void UART1putS (const char *Name2)				 
{
 char c;
 int i,len;
 
  len = strlen(Name2);
	
 for (i=0; i<len; i++) 	/* go through string */
  {
	c=(Name2[i]);			/* pick char */
	UART1putch (c);			/* send it out */
  }
}

/*****************************************************************************/
/*EOF $Id: uart1.c,v 1.1 2003/02/03 16:00:25 dfisch Exp $ */


